import type { USchema, MagicSuffix, DB_TYPE } from './types';
import type { TProperties, TPartial, TObject, StringOptions, StringFormatOption, DateOptions, NumericOptions } from '@sinclair/typebox';
export declare const NONE_PARAM: Set<"Min" | "MinThan" | "Max" | "MaxThan" | "MinH" | "MinD" | "MinM" | "MaxH" | "MaxD" | "MaxM" | "Like" | "Likel" | "Liker" | "Bt" | "BtD" | "BtY" | "BtM" | "Not" | "IsNull" | "NotNull" | "IsDistinct" | "NotDistinct" | ">" | ">=" | "<" | "<=" | "=" | "!=" | "<>" | "In" | "NotIn">;
export type Settings = {
    provider: [DB_TYPE, () => any] | (() => any);
    pageSize?: number;
    /**
     * A log Function Example : default :null
     *      console.info
     *      logger.log
     */
    showSQL?: Function;
    strict?: boolean | {
        query?: boolean;
        entity?: boolean;
    };
};
type UStringOptions<Format extends string> = USchema & StringOptions<Format> & {
    /**
     * Defind a mark:
     *      1. Delete action will update this filed to mark
     *      2. Query  action will add conditon with != ${delMark}
     *      3. Note : A table can only hava ONE delMark.
    */
    delMark?: string;
};
type UNumericOptions = USchema & {
    /**
     * Defind a mark:
     *      1. Delete action will update this filed to mark
     *      2. Query  action will add conditon with != ${delMark}
     *      3. Note : A table can only hava ONE delMark.
    */
    delMark?: number;
} & NumericOptions;
type UDateOptions = USchema & DateOptions & {
    /**
     * 1. Create Time can not be modify
     * 2. It will be auto fill with Current Time while INSERT
     * 3. default；: flase
    */
    isCreate?: boolean;
    /**
     * 1. Last Modify Time will be fill with Current Time while UPDATE
     * 2. default；: flase
    */
    isModify?: boolean;
};
export declare var ShowSql: any;
export declare var PAGE_SIZE: number;
export declare const setup: (settings: Settings) => void;
export declare const UType: {
    Table: <T extends TProperties>(properties: T) => TPartial<TObject<T>>;
    Number: (options?: UNumericOptions) => import("@sinclair/typebox").TNumber;
    String: <Format extends string>(options?: UStringOptions<StringFormatOption | Format>) => import("@sinclair/typebox").TString<Format>;
    Date: (options?: UDateOptions) => import("@sinclair/typebox").TDate;
    Boolean: (options?: USchema) => import("@sinclair/typebox").TBoolean;
    Integer: (options?: UNumericOptions) => import("@sinclair/typebox").TInteger;
};
export declare const throwErr: (err: string[], message?: string) => void;
export declare const checkEntity: (T: TObject, val: any) => void;
export declare const betweenNumber: (txt: string) => [MagicSuffix, number][];
export declare const betweenDate: (txt: string) => [MagicSuffix, Date][];
export declare const inString: (txt: string) => string[];
export declare const inNumber: (txt: string) => number[];
export declare const boolValue: (value: any) => boolean;
export {};
