import type { TObject, Static } from '@sinclair/typebox';
import type { QuerySchema, WhereParam, WhereDefine, USchema } from '../../base/types';
import type { TableOptions } from '../../base/BaseView';
import type { SearchRequest, SearchResponse, QueryDslQueryContainer } from '@elastic/elasticsearch/lib/api/types';
import { BaseQuery } from './BaseQuery';
export declare class FlatView<T extends TObject> extends BaseQuery {
    protected _index: string;
    private _F2C;
    private _C2F;
    protected _CONFIG: {
        sort: any;
        mark: any;
        fields_exclude: any[];
        pageSize: number;
        FIELD_MAP: Map<string, USchema>;
        globalFilter: QueryDslQueryContainer;
    };
    protected _QUERY_CACHE: Map<string, WhereDefine>;
    /**
     * @param tableName Data table name, "${schemaName}.${tableName}"
     *  "${schemaName}." can be ignore with the default search_path.
     * @param schema The Object Schema, oor will not validate the value
     * @param options (Table/View) Options
     *
    */
    constructor(indexName: string, schema: T, options?: TableOptions);
    all(): Promise<Static<T>[]>;
    private _query;
    /**
     * @see WhereCondition
     * Use a WhereCondition Query Data
    */
    queryByCondition(condition?: WhereParam, query?: QuerySchema): Promise<Static<T>[]>;
    /**
     * @see QuerySchema
     * Use a QuerySchema Query Data
    */
    query(query?: QuerySchema): Promise<Static<T>[]>;
    /**
     * Exec A Senctence
    */
    sql(request?: SearchRequest): Promise<SearchResponse<Static<T>>>;
    /**
     * @see QuerySchema
     * Use a QuerySchema Query Data With Page
     * this will return a object with {total:number,list:any[]}
    */
    queryPager(query?: QuerySchema): Promise<{
        total: number;
        list: Static<T>[];
    }>;
    protected byField(field: string, value?: string | number | boolean): QueryDslQueryContainer;
    /**
     * Get A record form Table / View By Primary key.
     * This method will return All column. Even if the IGNORE column.
    */
    getById(id: string): Promise<Static<T>>;
    /**
     * Get A record form Table / View By Specify Field = value.
     * This method will return All column. Even if the IGNORE column.
     * Note : If result has multi records , return the first row
     *        Want return all records?  use `queryByField`
    */
    getByField(field: string, value: string | number): Promise<Static<T>>;
    /**
     * Get records form Table / View By Specify Property = value.
    */
    queryByField(field: string, value?: string | number | boolean): Promise<Static<T>[]>;
    private orderByLimit;
}
