"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.FlatView = void 0;
const lodash_1 = __importDefault(require("lodash"));
const Util_1 = require("../../base/Util");
const BaseQuery_1 = require("./BaseQuery");
const QueryBuilder_1 = require("../../base/QueryBuilder");
const executor_1 = require("./executor");
const dsl_1 = require("./dsl");
const ES_MAX_SIZE = 10000;
class FlatView extends BaseQuery_1.BaseQuery {
    /**
     * @param tableName Data table name, "${schemaName}.${tableName}"
     *  "${schemaName}." can be ignore with the default search_path.
     * @param schema The Object Schema, oor will not validate the value
     * @param options (Table/View) Options
     *
    */
    constructor(indexName, schema, options) {
        super();
        this._F2C = new Map(); // Field To Column
        this._C2F = new Map(); // Column To Field
        this._CONFIG = {
            sort: null,
            mark: null,
            fields_exclude: [],
            pageSize: Util_1.PAGE_SIZE,
            FIELD_MAP: new Map(),
            globalFilter: null,
        };
        this._QUERY_CACHE = new Map();
        this._index = indexName;
        this._CONFIG.fields_exclude = [];
        this._CONFIG.FIELD_MAP = new Map();
        var WHERE = [];
        lodash_1.default.keys(schema.properties).map(field => {
            let properties = schema.properties[field];
            let column = properties.column || field;
            this._F2C.set(field, column);
            this._C2F.set(column, field);
            this._CONFIG.FIELD_MAP.set(field, properties);
            if (lodash_1.default.has(properties, 'delMark') && properties.delMark != null) {
                this._CONFIG.mark = { [column]: properties.delMark };
                WHERE.push({ field: column, value: properties.delMark, condition: '!=' });
            }
            if (properties.ignore) {
                this._CONFIG.fields_exclude.push(column);
                return;
            }
        });
        if (options == null) {
            this._CONFIG.globalFilter = (0, dsl_1.fixWhere)(WHERE);
            return;
        }
        if (options.key) {
            if (this._CONFIG.sort == null) {
                this._CONFIG.sort = { [options.key]: options.sortBy ? options.sortBy : 'desc' };
            }
        }
        if (options.sortOrder)
            this._CONFIG.sort = { [options.sortOrder]: options.sortBy ? options.sortBy : 'desc' };
        if (options.pageSize)
            this._CONFIG.pageSize = options.pageSize;
        if (options.globalCondition && options.globalCondition.length) {
            options.globalCondition.map(item => {
                let schema = this._CONFIG.FIELD_MAP.get(this._C2F.get(item.column));
                if (schema) {
                    WHERE.push(Object.assign(Object.assign({}, item), { type: (0, QueryBuilder_1.getFieldType)(schema) }));
                }
                else {
                    console.error(item);
                }
            });
        }
        ;
        this._CONFIG.globalFilter = (0, dsl_1.fixWhere)(WHERE);
    }
    ;
    all() {
        const orderBy = this.orderByLimit({ start_: 0, count_: ES_MAX_SIZE });
        return this._query(null, orderBy);
    }
    _query(param, orderBy) {
        return __awaiter(this, void 0, void 0, function* () {
            const { _CONFIG: { fields_exclude, globalFilter } } = this;
            const request = (0, dsl_1.buildSearch)(this._index, param, orderBy, fields_exclude, globalFilter);
            return executor_1.fxecutor.query(this.getClient(), request);
        });
    }
    /**
     * @see WhereCondition
     * Use a WhereCondition Query Data
    */
    queryByCondition(condition, query) {
        const { _CONFIG: { pageSize } } = this;
        const param = (0, dsl_1.where)(condition);
        let orderBy;
        if (query) {
            orderBy = this.orderByLimit(query);
        }
        else {
            orderBy = this.orderByLimit({ start_: 0, count_: pageSize });
        }
        return this._query(param, orderBy);
    }
    /**
     * @see QuerySchema
     * Use a QuerySchema Query Data
    */
    query(query) {
        const { _QUERY_CACHE, _CONFIG: { FIELD_MAP, } } = this;
        const condition = (0, QueryBuilder_1.queryToCondition)(query, FIELD_MAP, _QUERY_CACHE);
        return this.queryByCondition(condition, query);
    }
    /**
     * Exec A Senctence
    */
    sql(request) {
        return this.getClient().search(request);
    }
    /**
     * @see QuerySchema
     * Use a QuerySchema Query Data With Page
     * this will return a object with {total:number,list:any[]}
    */
    queryPager(query) {
        return __awaiter(this, void 0, void 0, function* () {
            const { _QUERY_CACHE, _CONFIG: { FIELD_MAP, fields_exclude, globalFilter } } = this;
            const condition = (0, QueryBuilder_1.queryToCondition)(query, FIELD_MAP, _QUERY_CACHE);
            const orderBy = this.orderByLimit(query);
            const param = (0, dsl_1.where)(condition);
            const request = (0, dsl_1.buildSearch)(this._index, param, orderBy, fields_exclude, globalFilter);
            return executor_1.fxecutor.queryPager(this.getClient(), request);
        });
    }
    byField(field, value) {
        const { _CONFIG: { globalFilter, FIELD_MAP }, _F2C, _C2F } = this;
        let column = _F2C.has(field) ? _F2C.get(field) : (_C2F.has(field) ? field : null);
        if (column == null) {
            throw new Error(`Index ${this._index} do not has field ${field}`);
        }
        let schema = FIELD_MAP.get(_C2F.get(column));
        const type = (0, QueryBuilder_1.getFieldType)(schema);
        return (0, dsl_1.where)([{ column, type, value }]);
    }
    /**
     * Get A record form Table / View By Primary key.
     * This method will return All column. Even if the IGNORE column.
    */
    getById(id) {
        return executor_1.fxecutor.getById(this.getClient(), this._index, id);
    }
    /**
     * Get A record form Table / View By Specify Field = value.
     * This method will return All column. Even if the IGNORE column.
     * Note : If result has multi records , return the first row
     *        Want return all records?  use `queryByField`
    */
    getByField(field, value) {
        const { _CONFIG: { globalFilter, } } = this;
        const param = this.byField(field, value);
        const request = (0, dsl_1.buildSearch)(this._index, param, null, [], globalFilter);
        return executor_1.fxecutor.getOne(this.getClient(), request);
    }
    /**
     * Get records form Table / View By Specify Property = value.
    */
    queryByField(field, value) {
        const param = this.byField(field, value);
        const orderBy = this.orderByLimit();
        return this._query(param, orderBy);
    }
    orderByLimit(query) {
        const { _CONFIG: { sort, pageSize, } } = this;
        let orderBy = {};
        if (query == null) {
            orderBy.from = 0;
            orderBy.size = pageSize;
            if (sort)
                orderBy.sort = sort;
            return orderBy;
        }
        orderBy.from = query.start_ || 0;
        orderBy.size = query.count_ || pageSize;
        if (this._F2C.has(query.order_)) {
            let by = lodash_1.default.trim(query.by_) == 'asc' ? 'asc' : 'desc';
            orderBy.sort = `${this._F2C.get(query.order_)}:${by}`;
        }
        return orderBy;
    }
}
exports.FlatView = FlatView;
