"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.Table = void 0;
const lodash_1 = __importDefault(require("lodash"));
const View_1 = require("./View");
const QueryBuilder_1 = require("../../base/QueryBuilder");
const executor_1 = require("./executor");
const dsl_1 = require("./dsl");
class Table extends View_1.View {
    /**
     * Auto convert data
     * check row data while insert or update
     * */
    checkEntity(obj, isAdd = false) {
        // checkEntity(this.schema)
        let clone = {};
        this._CONFIG.FIELD_MAP.forEach((schema, key) => {
            let field = schema.column || key;
            if (lodash_1.default.has(obj, key)) {
                clone[field] = obj[key];
            }
            let type = (0, QueryBuilder_1.getFieldType)(schema);
            if (type == 'date') {
                if (schema.isCreate) {
                    if (isAdd) {
                        clone[field] = new Date();
                    }
                    else {
                        lodash_1.default.unset(clone, field);
                    }
                    return;
                }
                if (schema.isModify) {
                    clone[field] = new Date();
                    return;
                }
            }
        });
        return clone;
    }
    // deleteByField(field: string, value: string | number | boolean): Promise<number> {
    //     const { _CONFIG: { mark, FIELD_MAP } } = this;
    //     if (mark) return this.updateByField(mark, field, value)
    //     let schema = FIELD_MAP.get(field);
    //     let column = (schema && schema.column) ? schema.column : field;
    //     return this.deleteByCondition([{ column, value }])
    // }
    // deleteByQuery(query: QuerySchema): Promise<number> {
    //     const { _CONFIG: { mark } } = this;
    //     if (mark) return this.updateByQuery(mark, query)
    //     const condition = queryToCondition(query, this._CONFIG.FIELD_MAP, this._QUERY_CACHE);
    //     return this.deleteByCondition(condition);
    // }
    deleteByCondition(condition) {
        const { _CONFIG: { globalFilter }, _index } = this;
        // const { _table, _BUILDER, _EXECUTOR, _CONFIG: { mark } } = this;
        // if (mark) return this.updateByCondition(mark, condition)
        // const SQL = _BUILDER.delete(_table);
        const param = (0, dsl_1.where)(condition);
        const query = (0, dsl_1.fixQuery)(globalFilter, param);
        return executor_1.executor.deleteByQuery(this.getClient(), _index, query);
    }
    deleteById(id) {
        const { _CONFIG: { mark } } = this;
        if (mark)
            return this.update(id, Object.assign({}, mark));
        return executor_1.executor.deleteById(this.getClient(), this._index, id);
    }
    /**
     * Update a record, By Primary Key in the obj
    */
    update(id, obj, useIndex = false) {
        const entity = this.checkEntity(obj, false);
        return executor_1.executor.updateById(this.getClient(), this._index, id, entity, useIndex);
    }
    // updateByField(obj: Static<T>, field: string, value: string | number | boolean): Promise<number> {
    //     let schema = this._CONFIG.FIELD_MAP.get(field);
    //     let column = (schema && schema.column) ? schema.column : field;
    //     return this.updateByCondition(obj, [{ column, value }])
    // }
    // updateByQuery(obj: Static<T>, query: QuerySchema): Promise<number> {
    //     const condition = queryToCondition(query, this._CONFIG.FIELD_MAP, this._QUERY_CACHE);
    //     return this.updateByCondition(obj, condition);
    // }
    // updateByCondition(obj: Static<T>, condition?: WhereParam): Promise<number> {
    //     const { _table, _BUILDER, _EXECUTOR, _CONFIG: { key } } = this;
    //     _.unset(obj, key);
    //     let entity = this.checkEntity(obj, false);
    //     if (_.keys(entity).length == 0) return new Promise(r => r(0));
    //     const [SQL, FIELD_SET] = _BUILDER.update(_table, entity);
    //     const [WHERE, PARAM] = this._BUILDER.where(condition, FIELD_SET.length + 1);
    //     return _EXECUTOR.execute(this.getClient(), `${SQL} ${this.fixWhere(WHERE)}`, [...FIELD_SET, ...PARAM]);
    // }
    /**
     * Insert a record
    */
    insert(object) {
        let entity = this.checkEntity(object, true);
        return executor_1.executor.add(this.getClient(), this._index, entity);
    }
}
exports.Table = Table;
